package java.security;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.util.Hashtable;
import java.io.ObjectOutputStream;
import java.util.List;
import java.security.cert.Certificate;
import java.util.Enumeration;
import java.util.HashMap;
import java.io.ObjectStreamField;
import java.util.Map;
import java.io.Serializable;

public final class Permissions extends PermissionCollection implements Serializable
{
    private transient Map<Class<?>, PermissionCollection> permsMap;
    private transient boolean hasUnresolved;
    PermissionCollection allPermission;
    private static final long serialVersionUID = 4858622370623524688L;
    private static final ObjectStreamField[] serialPersistentFields;
    
    public Permissions() {
        this.hasUnresolved = false;
        this.permsMap = new HashMap(11);
        this.allPermission = null;
    }
    
    @Override
    public void add(Permission permission) {
        if (this.isReadOnly()) {
            throw new SecurityException("attempt to add a Permission to a readonly Permissions object");
        }
        PermissionCollection permissionCollection;
        synchronized (this) {
            permissionCollection = this.getPermissionCollection(permission, true);
            permissionCollection.add(permission);
        }
        if ((permission instanceof AllPermission)) {
            this.allPermission = permissionCollection;
        }
        if ((permission instanceof UnresolvedPermission)) {
            this.hasUnresolved = true;
        }
        return;
    }
    
    @Override
    public boolean implies(Permission permission) {
        if (this.allPermission != null) {
            return true;
        }
        synchronized (this) {
            PermissionCollection permissionCollection = this.getPermissionCollection(permission, false);
            if (permissionCollection != null) {
                return permissionCollection.implies(permission);
            }
            return false;
        }
    }
    
    @Override
    public Enumeration<Permission> elements() {
        synchronized (this) {
            return new PermissionsEnumerator(this.permsMap.values().iterator());
        }
    }
    
    private PermissionCollection getPermissionCollection(Permission permission, boolean b) {
        Class<Object> class1 = permission.getClass();
        PermissionCollection permissionCollection = (PermissionCollection)this.permsMap.get(class1);
        if (!this.hasUnresolved && !b) {
            return permissionCollection;
        }
        if (permissionCollection == null) {
            permissionCollection = (this.hasUnresolved ? this.getUnresolvedPermissions(permission) : null);
            if (permissionCollection == null && b) {
                permissionCollection = permission.newPermissionCollection();
                if (permissionCollection == null) {
                    permissionCollection = new PermissionsHash();
                }
            }
            if (permissionCollection != null) {
                this.permsMap.put(class1, permissionCollection);
            }
        }
        return permissionCollection;
    }
    
    private PermissionCollection getUnresolvedPermissions(Permission permission) {
        UnresolvedPermissionCollection collection = (UnresolvedPermissionCollection)this.permsMap.get(UnresolvedPermission.class);
        if (collection == null) {
            return null;
        }
        List<UnresolvedPermission> unresolvedPermissions = collection.getUnresolvedPermissions(permission);
        if (unresolvedPermissions == null) {
            return null;
        }
        Certificate[] array = null;
        Object[] signers = permission.getClass().getSigners();
        int n = 0;
        if (signers != null) {
            for (int i = 0; i < signers.length; ++i) {
                if ((signers[i] instanceof Certificate)) {
                    ++n;
                }
            }
            array = new Certificate[n];
            int n2 = 0;
            for (int j = 0; j < signers.length; ++j) {
                if ((signers[j] instanceof Certificate)) {
                    array[n2++] = (Certificate)signers[j];
                }
            }
        }
        Object permissionCollection = null;
        synchronized (unresolvedPermissions) {
            for (int size = unresolvedPermissions.size(), k = 0; k < size; ++k) {
                Permission resolve = ((UnresolvedPermission)unresolvedPermissions.get(k)).resolve(permission, array);
                if (resolve != null) {
                    if (permissionCollection == null) {
                        permissionCollection = permission.newPermissionCollection();
                        if (permissionCollection == null) {
                            permissionCollection = new PermissionsHash();
                        }
                    }
                    permissionCollection.add(resolve);
                }
            }
        }
        return permissionCollection;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        Hashtable<Object, Object> hashtable = new Hashtable(this.permsMap.size() * 2);
        synchronized (this) {
            hashtable.putAll(this.permsMap);
        }
        ObjectOutputStream.PutField putFields = objectOutputStream.putFields();
        putFields.put("allPermission", this.allPermission);
        putFields.put("perms", hashtable);
        objectOutputStream.writeFields();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        ObjectInputStream.GetField fields = objectInputStream.readFields();
        this.allPermission = (PermissionCollection)fields.get("allPermission", null);
        Hashtable<Object, Object> hashtable = (Hashtable)fields.get("perms", null);
        (this.permsMap = new HashMap(hashtable.size() * 2)).putAll(hashtable);
        UnresolvedPermissionCollection collection = (UnresolvedPermissionCollection)this.permsMap.get(UnresolvedPermission.class);
        this.hasUnresolved = (collection != null && collection.elements().hasMoreElements());
    }
    
    static {
        serialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("perms", Hashtable.class), new ObjectStreamField("allPermission", PermissionCollection.class) };
    }
}
